==> hermes-5.5.0.11.tar.gz: hermes-5.5.0.11/lib/hermes/bo_client/client.rb <==
require 'rest-client'
require 'base64'
require 'nokogiri'

module Hermes
  class BoClient::Client
    class ClientError < StandardError
      attr_reader :response
      
      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"
        
        super(message)
      end
      
      def code
        to_xml&.at_xpath('code')&.text
      end
      
      def error_message
        to_xml&.at_xpath('message')&.text
      end
      
      def to_xml
        Nokogiri::XML(response.to_s).at_xpath('//errorResponse')
      end
      
      def request
        response.request
      end
    end
    
    class Forbidden < StandardError;
    end
    class NotFound < StandardError;
    end
    
    attr_accessor :base_url
    attr_accessor :user
    attr_accessor :password
    attr_accessor :logger
    
    def initialize(attrs = {})
      @base_url = attrs.fetch(:base_url, BoClient.config.base_url)
      @user = attrs.fetch(:user, BoClient.config.user)
      @password = attrs.fetch(:password, BoClient.config.password)
      @logger = attrs.fetch(:logger, BoClient.config.logger)
    end
    
    def log_error(error, path, params, classname = nil, id = nil)
      return unless classname && id
    
      instance = classname.constantize.find(id)
      return unless instance
      instance.add_error(error, path, params)
    end
    
    def log_success(classname = nil, id = nil)
      return unless classname && id
      
      instance = classname.constantize.find(id)
      return unless instance
      instance.set(last_send_has_error:false, updated_at: DateTime.current) if instance.last_send_has_error
    end
    
    def endpoint_for(path)
      base_resource[path]
    end
    
    def get(path, params = {}, classname = nil, id = nil)
      Nokogiri::XML(raw_get(path, params, classname = nil, id = nil))
    end
    
    def raw_get(path, params = {}, classname = nil, id = nil)
      begin
        endpoint_for(path).get(params)
        log_success(classname, id )
      rescue RestClient::ExceptionWithResponse => err
        log_error("ExceptionWithResponse ----> "+err.response.to_s.force_encoding('UTF-8'), path, params, classname, id)
        false
        #raise ClientError.new(err.response)
      end
    end
    
    def post(path, classname, id, payload, params = {})
      begin
        e = endpoint_for(path)
        
        if @logger
          @logger.debug "POST #{e.url}"
          @logger.debug "Payload : #{payload}"
        end
        Hermes::LogJob.perform_later(level: 'info', code: "BO_CLIENT_POST", action: "#{classname} : #{id} -- POST #{e.url}", data: { params: params, payload: payload }.as_json.transform_values{|v| v.to_s})
        r = e.post(payload, params)
        
        @logger&.debug "Response:  #{r}"
        log_success(classname, id)
        Hermes::LogJob.perform_later(level: 'info', code: "BO_CLIENT_POST", action: "#{classname} : #{id} -- POST #{e.url} SUCCESS", data: { response: r }.as_json.transform_values{|v| v.to_s})
        r
      rescue RestClient::ResourceNotFound => e
        log_error("ResourceNotFound ----> "+ e.response.to_s.force_encoding('UTF-8'), path, payload,classname, id)
        Hermes::LogJob.perform_later(level: 'error', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- ResourceNotFound", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
        #raise NotFound
      rescue RestClient::Forbidden => e
        log_error("Forbidden ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        Hermes::LogJob.perform_later(level: 'error', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- Forbidden", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
        ##raise Forbidden
      rescue RestClient::ExceptionWithResponse => e
        if @logger
          @logger.debug "Response:  #{e.response.to_s.force_encoding('UTF-8')}"
        end
        log_error("ExceptionWithResponse ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- ExceptionWithResponse", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
        ##raise ClientError.new(e.response.force_encoding('UTF-8'))
      rescue => e
        if @logger
          @logger.debug "Response:  #{e.inspect}"
        end
        log_error("UndefinedError ----> "+e.inspect, path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- UndefinedError", data: { response: e.inspect }.as_json)
        false
      end
    end

    def put(path, classname, id, payload, params = {})
      begin
        e = endpoint_for(path)
    
        if @logger
          @logger.debug "PUT #{e.url}"
          @logger.debug "Payload : #{payload}"
        end
        Log.perform_later(level: 'info', domain: 'api', code: "BO_CLIENT_PUT", action: "#{classname} : #{id} -- PUT #{e.url}", data: { params: params, payload: payload }.as_json.transform_values{|v| v.to_s})
        r = e.put(payload, params)
    
        @logger&.debug "Response:  #{r}"
        log_success(classname, id )
        Log.perform_later(level: 'info', domain: 'api', code: "BO_CLIENT_PUT", action: "#{classname} : #{id} -- PUT #{e.url} SUCCESS", data: { response: r }.as_json.transform_values{|v| v.to_s})
        r.to_s
      rescue RestClient::ResourceNotFound => e
        log_error("ResourceNotFound ----> "+ e.response.to_s.force_encoding('UTF-8'), path, payload,classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- ResourceNotFound", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
          #raise NotFound
      rescue RestClient::Forbidden => e
        log_error("Forbidden ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- Forbidden", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
          ##raise Forbidden
      rescue RestClient::ExceptionWithResponse => e
        if @logger
          @logger.debug "Response:  #{e.response.to_s.force_encoding('UTF-8')}"
        end
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- ExceptionWithResponse", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        log_error("ExceptionWithResponse ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        false
        ##raise ClientError.new(e.response.force_encoding('UTF-8'))
      rescue => e
        if @logger
          @logger.debug "Response:  #{e.inspect}"
        end
        log_error("UndefinedError ----> "+e.inspect, path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- UndefinedError", data: { response: e.inspect }.as_json)
        false
      end
    end

    def base_resource(reload = false)
      if !@base_resource || reload
        auth_header = 'Basic ' + Base64.encode64("#{@user}:#{@password}").chomp
        @base_resource = RestClient::Resource.new(
          @base_url,
          headers: {authorization: auth_header, 'Content-Type' => 'application/xml'},
          log: logger,
          verify_ssl: false
        )
      end
      @base_resource
    end
  end
end

==> hermes-5.6.0.12.tar.gz: hermes-5.6.0.12/lib/hermes/bo_client/client.rb <==
require 'rest-client'
require 'base64'
require 'nokogiri'

module Hermes
  class BoClient::Client
    class ClientError < StandardError
      attr_reader :response
      
      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"
        
        super(message)
      end
      
      def code
        to_xml&.at_xpath('code')&.text
      end
      
      def error_message
        to_xml&.at_xpath('message')&.text
      end
      
      def to_xml
        Nokogiri::XML(response.to_s).at_xpath('//errorResponse')
      end
      
      def request
        response.request
      end
    end
    
    class Forbidden < StandardError;
    end
    class NotFound < StandardError;
    end
    
    attr_accessor :base_url
    attr_accessor :user
    attr_accessor :password
    attr_accessor :logger
    
    def initialize(attrs = {})
      @base_url = attrs.fetch(:base_url, BoClient.config.base_url)
      @user = attrs.fetch(:user, BoClient.config.user)
      @password = attrs.fetch(:password, BoClient.config.password)
      @logger = attrs.fetch(:logger, BoClient.config.logger)
    end
    
    def log_error(error, path, params, classname = nil, id = nil)
      return unless classname && id
    
      instance = classname.constantize.find(id)
      return unless instance
      instance.add_error(error, path, params)
    end
    
    def log_success(classname = nil, id = nil)
      return unless classname && id
      
      instance = classname.constantize.find(id)
      return unless instance
      instance.set(last_send_has_error:false, updated_at: DateTime.current) if instance.last_send_has_error
    end
    
    def endpoint_for(path)
      base_resource[path]
    end
    
    def get(path, params = {}, classname = nil, id = nil)
      Nokogiri::XML(raw_get(path, params, classname = nil, id = nil))
    end
    
    def raw_get(path, params = {}, classname = nil, id = nil)
      begin
        endpoint_for(path).get(params)
        log_success(classname, id )
      rescue RestClient::ExceptionWithResponse => err
        log_error("ExceptionWithResponse ----> "+err.response.to_s.force_encoding('UTF-8'), path, params, classname, id)
        false
        #raise ClientError.new(err.response)
      end
    end
    
    def post(path, classname, id, payload, params = {})
      begin
        e = endpoint_for(path)
        
        if @logger
          @logger.debug "POST #{e.url}"
          @logger.debug "Payload : #{payload}"
        end
        Hermes::LogJob.add_to_queue(level: 'info', code: "BO_CLIENT_POST", action: "#{classname} : #{id} -- POST #{e.url}", data: { params: params, payload: payload }.as_json.transform_values{|v| v.to_s})
        r = e.post(payload, params)
        
        @logger&.debug "Response:  #{r}"
        log_success(classname, id)
        Hermes::LogJob.add_to_queue(level: 'info', code: "BO_CLIENT_POST", action: "#{classname} : #{id} -- POST #{e.url} SUCCESS", data: { response: r }.as_json.transform_values{|v| v.to_s})
        r
      rescue RestClient::ResourceNotFound => e
        log_error("ResourceNotFound ----> "+ e.response.to_s.force_encoding('UTF-8'), path, payload,classname, id)
        Hermes::LogJob.add_to_queue(level: 'error', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- ResourceNotFound", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
        #raise NotFound
      rescue RestClient::Forbidden => e
        log_error("Forbidden ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        Hermes::LogJob.add_to_queue(level: 'error', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- Forbidden", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
        ##raise Forbidden
      rescue RestClient::ExceptionWithResponse => e
        if @logger
          @logger.debug "Response:  #{e.response.to_s.force_encoding('UTF-8')}"
        end
        log_error("ExceptionWithResponse ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- ExceptionWithResponse", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
        ##raise ClientError.new(e.response.force_encoding('UTF-8'))
      rescue => e
        if @logger
          @logger.debug "Response:  #{e.inspect}"
        end
        log_error("UndefinedError ----> "+e.inspect, path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_POST_ERROR", action: "#{classname} : #{id} -- UndefinedError", data: { response: e.inspect }.as_json)
        false
      end
    end

    def put(path, classname, id, payload, params = {})
      begin
        e = endpoint_for(path)
    
        if @logger
          @logger.debug "PUT #{e.url}"
          @logger.debug "Payload : #{payload}"
        end
        Log.perform_later(level: 'info', domain: 'api', code: "BO_CLIENT_PUT", action: "#{classname} : #{id} -- PUT #{e.url}", data: { params: params, payload: payload }.as_json.transform_values{|v| v.to_s})
        r = e.put(payload, params)
    
        @logger&.debug "Response:  #{r}"
        log_success(classname, id )
        Log.perform_later(level: 'info', domain: 'api', code: "BO_CLIENT_PUT", action: "#{classname} : #{id} -- PUT #{e.url} SUCCESS", data: { response: r }.as_json.transform_values{|v| v.to_s})
        r.to_s
      rescue RestClient::ResourceNotFound => e
        log_error("ResourceNotFound ----> "+ e.response.to_s.force_encoding('UTF-8'), path, payload,classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- ResourceNotFound", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
          #raise NotFound
      rescue RestClient::Forbidden => e
        log_error("Forbidden ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- Forbidden", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        false
          ##raise Forbidden
      rescue RestClient::ExceptionWithResponse => e
        if @logger
          @logger.debug "Response:  #{e.response.to_s.force_encoding('UTF-8')}"
        end
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- ExceptionWithResponse", data: { response: e.response.to_s.force_encoding('UTF-8') }.as_json)
        log_error("ExceptionWithResponse ----> "+e.response.to_s.force_encoding('UTF-8'),path, payload, classname, id)
        false
        ##raise ClientError.new(e.response.force_encoding('UTF-8'))
      rescue => e
        if @logger
          @logger.debug "Response:  #{e.inspect}"
        end
        log_error("UndefinedError ----> "+e.inspect, path, payload, classname, id)
        Log.perform_later(level: 'error', domain: 'api', code: "BO_CLIENT_PUT_ERROR", action: "#{classname} : #{id} -- UndefinedError", data: { response: e.inspect }.as_json)
        false
      end
    end

    def base_resource(reload = false)
      if !@base_resource || reload
        auth_header = 'Basic ' + Base64.encode64("#{@user}:#{@password}").chomp
        @base_resource = RestClient::Resource.new(
          @base_url,
          headers: {authorization: auth_header, 'Content-Type' => 'application/xml'},
          log: logger,
          verify_ssl: false
        )
      end
      @base_resource
    end
  end
end
